<?php
// wcf imports
require_once(WCF_DIR.'lib/util/pip/XMLPIP.class.php');

/**
 * Basic implementation to create generic XML-PIPs
 *
 * @package	com.woltlab.community.wcf.util.package
 * @author	Alexander Ebert
 * @copyright	2010 WoltLab Community
 * @license	GNU Lesser Public License <http://www.gnu.org/licenses/lgpl.html>
 * @subpackage	lib.util.pip
 */
abstract class AbstractXMLPIP implements XMLPIP {
	/**
	 * Creates a generic xml string
	 *
	 * @param	mixed		$tagName		XML-PIP name
	 * @param	array<array>	$data			XML-PIP data
	 * @param	array		$validElements		Verifies data node elements against an array
	 * @param	boolean		$appendParentCategory	Appends category to each node name
	 * @param	string		$container		Defaults to "data". Main node for each XML-PIP
	 * @param	array		$additionalAttributes	Appends attributes to $tagName data node
	 * @param	boolean		$useGenericXMLDataNode	Defaults to false. If true uses genericl xml data nodes instead of import/delete nodes
	 * @param	array<array>	$validAttributes	Allows deeper xml nodes for item-specific attribute elements
	 * @return	string		Generic xml string
	 */
	protected static function createGenericXML ($tagName, &$data, $validElements = null, $appendParentCategory = false, $validAttributes = null, $useGenericXMLDataNode = false, $container = 'data', $additionalAttributes = array()) {
		$attributes = '';

		// build additional attributes
		if (!empty($additionalAttributes)) {
			foreach ($additionalAttributes as $attribute => $value) {
				$attributes .= ' '.$attribute.'="'.$value.'"';
			}
		}

		// determine xsd and correct tagname
		if (is_array($tagName)) {
			$xsd = $tagName['xsd'];
			$tagName = $tagName['item'];
		}
		else {
			$xsd = (substr($tagName, -4) == 'item') ? substr($tagName, 0, -4) : $tagName;
		}

		// xml header with xsd
		$xml = '<?xml version="1.0" encoding="UTF-8"?>
<'.$container.$attributes.' xmlns="http://www.woltlab.com" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://www.woltlab.com/XSD/'.$xsd.'.xsd">';

		// handle generic import
		if (isset($data['import'])) {
			$xml .= "\n\t<import>";

			if ($useGenericXMLDataNode) {
				self::createGenericXMLDataNode($tagName, $data['import'], $xml, $validElements);
			}
			else {
				self::createGenericXMLImport($tagName, $data, $xml, $validElements, $appendParentCategory, $validAttributes);
			}

			$xml .= "\n\t</import>";
		}

		// handle generic delete
		if (isset($data['delete'])) {
			$xml .= "\n\t<delete>";

			if ($useGenericXMLDataNode) {
				self::createGenericXMLDataNode($tagName, $data['delete'], $xml, $validElements);
			}
			else {
				self::createGenericXMLDelete($tagName, $data, $xml);
			}

			$xml .= "\n\t</delete>";
		}

		// xml footer
		$xml .= "\n</".$container.">";

		return $xml;
	}

	/**
	 * Creates a xml data node used for import and delete actions
	 *
	 * @param	string	$tagName
	 * @param	array<array>	$data
	 * @param	string		$xml
	 * @param	array		$validElements
	 */
	protected static function createGenericXMLDataNode ($tagName, &$data, &$xml, $validElements) {
		foreach ($data as $elements) {
			$xml .= "\n\t\t<".$tagName.">";

			foreach ($elements as $name => $value) {
				$name = StringUtil::toLowerCase($name);

				if (!in_array($name, $validElements)) continue;
    				$xml .= "\n\t\t\t<".$name."><![CDATA[".$value."]]></".$name.">";
			}

			$xml .= "\n\t\t</".$tagName.">";
		}
	}

	/**
	 * Creates a xml delete node
	 *
	 * @param	string		$tagName
	 * @param	array<array>	$data
	 * @param	string		$xml
	 */
	protected static function createGenericXMLDelete ($tagName, &$data, &$xml) {
		// create item for each entry
  		foreach ($data['delete'] as $name) {
  			$xml .= "\n\t\t<".$tagName." name=\"".$name."\" />";
  		}
	}

	/**
	 * Creates a xml import node
	 *
	 * @param	string		$tagName
	 * @param	array<array>	$data
	 * @param	string		$xml
	 * @param	array		$validateElements
	 */
	protected static function createGenericXMLImport ($tagName, &$data, &$xml, &$validElements, $appendParentCategory, &$validAttributes) {
		// create a node for each element
		foreach ($data['import'] as $name => $elements) {
			// append parent category if requested
			if ($appendParentCategory && isset($elements['parent'])) {
				$name = $elements['parent'].'.'.$name;
			}

			// insert name attribute if available
			$name = (!is_numeric($name) && !empty($name)) ? ' name="'.$name.'"' : '';
			$xml .= "\n\t\t<".$tagName.$name.">";

			foreach ($elements as $element => $value) {
				$element = StringUtil::toLowerCase($element);

				// skip unknown elements
				if (($validElements !== null) && !in_array($element, $validElements));

				// insert element
				$xml .= "\n\t\t\t<".$element.">";

				if (($validAttributes !== null) && isset($validAttributes[$element])) {
					foreach ($value as $attributeName => $attributes) {
						$attributeName = StringUtil::toLowerCase($attributeName);
						$xml .= "\n\t\t\t\t<attribute name=\"".$attributeName."\">";

						foreach ($attributes as $attribute => $attributeValue) {
							// ignore unknown attributes
							if (!in_array($attribute, $validAttributes[$element])) continue;
							$xml .= "\n\t\t\t\t\t<".$attribute."><![CDATA[".$attributeValue."]]></".$attribute.">";
						}

						$xml .= "\n\t\t\t\t</attribute>";
					}

					$xml .= "\n\t\t\t</".$element.">";
				}
				else {
					$xml .= "<![CDATA[".$value."]]></".$element.">";
				}
			}

			// close node
			$xml .= "\n\t\t</".$tagName.">";
		}
	}
}
?>