<?php
// wcf imports
require_once(WCF_DIR.'lib/util/pip/AbstractXMLPIP.class.php');

/**
 * Creates an options XML-PIP
 *
 * @package	com.woltlab.community.wcf.util.package
 * @author	Alexander Ebert
 * @copyright	2010 WoltLab Community
 * @license	GNU Lesser Public License <http://www.gnu.org/licenses/lgpl.html>
 * @subpackage	lib.util.pip
 */
class OptionsXMLPIP extends AbstractXMLPIP {
	/**
	 * Stores all available option data elements and saves wether they may used in groupoptions
	 *
	 * @var	array
	 */
	public static $optionElements = array(
		'categoryname' => true,
		'enableoptions' => true,
		'defaultvalue' => true,
		'hidden' => false,
		'optiontype' => true,
		'selectoptions' => false,
		'showorder' => true,
		'validationpattern' => true
	);

	/**
	 * Returns a valid options xml string
	 *
	 * Usage:
	 * OptionsXMLPIP::create(array(
	 * 	'import' => array(
	 * 		'categories' => array(
	 * 			'foo' => 'admin',
	 * 			'bar' => 'admin.foo'
	 * 		),
	 * 		'options' => array(
	 * 			'test' => array(
	 * 				'categoryname' => 'admin.foo.bar',
	 * 				'optiontype' => 'boolean',
	 * 				'defaultvalue' => '0'
	 * 			)
	 * 		)
	 * 	),
	 * 	'delete' => array(
	 * 		'optioncategory' => array(
	 * 			'admin.foo.bar',
	 * 			'admin.foo'
	 * 		),
	 * 		'options' => array(
	 * 			'admin.foo.bar.test'
	 * 		)
	 * 	)
	 * ));
	 */
	public static function create ($data, $appendParentCategory = false) {
		return self::createOptionsXML('options', $data, $appendParentCategory, self::$optionElements);
	}

	/**
	 * @param	array<array>	$data			Groupoption categories and options
	 * @param	boolean		$appendParentGroup	Defaults to true. Prefixes categories and options with parent category
	 * @param	array<array>	$validElements		Lists all valid elements for import and delete nodes
	 * @return	string		Groupoptions XML string
	 * @see		OptionXMLPIP::create()
	 * @see		OptionXMLPIP::createOptionsXMLImport()
	 * @see		OptionXMLPIP::createOptionsXMLDelete()
	 */
	protected static function createOptionsXML ($type, $data, $appendParentCategory = true, $validElements) {
		// xml header with xsd
		$xml = '<?xml version="1.0" encoding="UTF-8"?>
<data xmlns="http://www.woltlab.com" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://www.woltlab.com/XSD/'.$type.'.xsd">';

		// handle import
		if (isset($data['import'])) {
			self::createOptionsXMLImport($data, $xml, $appendParentCategory, $validElements);
		}

		// handle delete
		if ($data['delete']) {
   			self::createOptionsXMLDelete($data, $xml);
		}

		// xml footer
		$xml .= "\n</data>";

		return $xml;
	}

	/**
	 * Creates the import node for groupoptions
	 *
	 * @param	boolean		$createGroupOptions	Determines wether groupoptions should be created
	 * @param	array<array>	$data			Reference to current option data
	 * @param	string		$xml			Reference to current xml string
	 * @param	boolean		$appendParentCategory	Defaults to true. Prefixes categories and options with parent category
	 */
	protected static function createOptionsXMLImport (&$data, &$xml, $appendParentCategory, $validElements) {
		$xml .= "\n\t<import>";

		// first insert categories
		if (isset($data['import']['categories'])) {
			$xml .= "\n\t\t<categories>";

			// create node for each category
			foreach ($data['import']['categories'] as $categoryName => $parentCategory) {
				if ($appendParentCategory && !empty($parentCategory)) $categoryName = $parentCategory.'.'.$categoryName;
				$xml .= "\n\t\t\t<category name=\"".$categoryName."\">\n";

    				// append parent entry if available
    				if (!empty($parentCategory)) $xml .= "\t\t\t\t<parent>".$parentCategory."</parent>\n";

    				$xml .= "\t\t\t</category>";
			}

			$xml .= "\n\t\t</categories>";
		}

		// last insert options
		if (isset($data['import']['options'])) {
			$xml .= "\n\t\t<options>";

			// create node for each option
			foreach ($data['import']['options'] as $optionName => $optionData) {
				if (!isset($optionData['categoryname']) || empty($optionData['categoryname'])) continue;

    				if ($appendParentCategory) $optionName = $optionData['categoryname'].'.'.$optionName;
    				$xml .= "\n\t\t\t<option name=\"".$optionName."\">";

				// create node for each option data entry
    				foreach ($optionData as $element => $value) {
    					$element = StringUtil::toLowerCase($element);

    					// ignore invalid data elements
    					if (!in_array($element, $validElements)) continue;
    					$xml .= "\n\t\t\t\t<".$element."><![CDATA[".$value."]]></".$element.">";
    				}

    				$xml .= "\n\t\t\t</option>";
			}

			$xml .= "\n\t\t</options>";
		}

		$xml .= "\n\t</import>";
	}

	/**
	 * Creates the delete node for options
	 *
	 * @param	array<array>	$data	Reference to current option data
	 * @param	string		$xml	Reference to current xml string
	 */
	protected static function createOptionsXMLDelete (&$data, &$xml) {
		$xml .= "\n\t<delete>";

		// first insert option categories to be removed
		if (isset($data['delete']['optioncategory'])) {
   			foreach ($data['delete']['optioncategory'] as $option) {
  				$xml .= "\n\t\t<optioncategory name=\"".$option."\" />";
			}
   		}

		// last insert options to be removed
		if (isset($data['delete']['option'])) {
   			foreach ($data['delete']['option'] as $option) {
   				$xml .= "\n\t\t<option name=\"".$option."\" />";
   			}
   		}

		$xml .= "\n\t</delete>";
	}
}
?>